from paircd.reply import (
    ERR_NOSUCHCHANNEL,
    ERR_TOOMANYCHANNELS,
    JOIN,
    RPL_ENDOFNAMES,
    RPL_NAMREPLY,
    RPL_NOTOPIC,
    RPL_TOPIC,
)

from paircd.client import Client
from paircd.command_handler import CommandHandler
from paircd.message import Message
from paircd.server import Server
from paircd.settings import USER_MAX_CHANNELS


class JoinHandler(CommandHandler):
    def __init__(self) -> None:
        super().__init__("JOIN", 1)

    async def handle(self, server: Server, client: Client, msg: Message) -> None:
        channel_name = msg.args[0]

        if USER_MAX_CHANNELS > 0 and len(client.channels) == USER_MAX_CHANNELS:
            client.write_message(ERR_TOOMANYCHANNELS(client.nickname))
            return

        channel = server.get_channel_by_name(channel_name)
        if channel is None:
            client.write_message(ERR_NOSUCHCHANNEL(client.nickname, channel_name))
            return

        channel.add_client(client)

        client.channels.add(channel_name)
        client.log(f"joined {channel_name}")

        channel.write_message(JOIN(channel_name, prefix=client.id()))

        if len(channel.topic) > 0:
            client.write_message(
                RPL_TOPIC(client.nickname, channel_name, channel.topic)
            )
        else:
            client.write_message(RPL_NOTOPIC(client.nickname, channel_name))

        channel_members = [
            channel.get_channel_name(member)
            for member in channel.clients_by_nick.keys()
        ]

        client.write_message(
            # "=" means public channel (ref: https://modern.ircdocs.horse/#rplnamreply-353)
            RPL_NAMREPLY(client.nickname, "=", channel_name, " ".join(channel_members))
        )
        client.write_message(RPL_ENDOFNAMES(client.nickname, channel_name))
