from paircd.reply import ERR_USERSDONTMATCH, MODE, RPL_CHANNELMODEIS, RPL_UMODEIS

from paircd.client import Client
from paircd.command_handler import CommandHandler
from paircd.message import Message
from paircd.server import Server


class ModeHandler(CommandHandler):
    def __init__(self) -> None:
        super().__init__("MODE")

    async def handle(self, server: Server, client: Client, msg: Message) -> None:
        name = msg.args[0]

        if name.startswith("#"):
            self.handle_channel(server, client, msg)
        else:
            self.handle_user(client, msg)

    def handle_user(self, client: Client, msg: Message) -> None:
        name = msg.args[0]

        if name != client.nickname:
            client.write_message(ERR_USERSDONTMATCH(client.nickname))
            return

        # Set user mode
        if len(msg.args) == 2:
            client.log("TODO: implement user mode set")

        settings = client.get_mode_settings()
        client.write_message(RPL_UMODEIS(client.nickname, settings))

    def handle_channel(self, server: Server, client: Client, msg: Message) -> None:
        name = msg.args[0]

        channel = server.get_channel_by_name(name, create=False)
        if channel is None:
            return

        if len(msg.args) == 2:
            client.log("TODO: implement channel mode set")

        modes = "".join(channel.get_modes())
        if modes != "":
            modes = f"+{modes}"

        client.write_message(MODE(name, modes))
        client.write_message(RPL_CHANNELMODEIS(client.nickname, name, modes, ""))
