from paircd.client import Client
from paircd.command_handler import CommandHandler
from paircd.message import Message
from paircd.reply import ERR_NICKNAMEINUSE, ERR_NONICKNAMEGIVEN, NICK
from paircd.server import Server


class NickHandler(CommandHandler):
    def __init__(self) -> None:
        super().__init__("NICK", 1)

    async def handle(self, server: Server, client: Client, msg: Message) -> None:
        nickname = msg.args[0]
        if nickname == "":
            target = client.nickname if client.nickname else nickname
            client.write_message(ERR_NONICKNAMEGIVEN(target))
            return

        existing_client = server.get_client_by_name(nickname)
        if existing_client is not None:
            if client != existing_client:
                target = client.nickname if client.nickname else nickname
                client.write_message(ERR_NICKNAMEINUSE(target, nickname))
            return

        rename = client.nickname != ""
        old_nick = client.nickname
        old_id = client.id()

        client.nickname = nickname
        if rename:
            server.rename_client(old_nick, nickname)
        else:
            server.add_client(client)

        for channel_name in client.channels:
            channel = server.get_channel_by_name(channel_name, create=False)
            if channel is None:
                continue

            if rename:
                channel.rename_client(old_nick, nickname)
                channel.write_message(NICK(nickname, prefix=old_id))
            else:
                raise RuntimeError("shouldn't be in any channels without a nick")

        client.register()
